#!/usr/sbin/perl -w
#
# GeneCount.pl v1.0a
# Last modified April 29, 2004 11:16
# (c) Olaf R.P. Bininda-Emonds
#
# Input: A GenBank summary file (GenBank.txt)
#
# Output: A quick and dirty count of how many times a given gene appears in the file;
#         there is no correction made for synonyms
#
# Usage: GeneCount.pl
#   no options

$geneFile = "GenBank.txt";	# Do search in GenBank using txid9263[Organism:exp]

setLineBreak($geneFile);
open (GENE, "<$geneFile") or die "Cannot open GenBank output file, $geneFile.\n";
	while (<GENE>)
		{
		chomp;
		$genLine = $_;
		if ($genLine =~ "LOCUS")
			{
			undef %iGene;
			}
		if ($genLine =~ "\/gene")
			{
			$gene = $genLine;
				$gene =~ s/"//g;
				$gene =~ s/^\s+//;
				$gene =~ s/\/gene=//;
			$iGene{$gene}++;
			$count{$gene}++ if ($iGene{$gene} == 1);
			push @GeneList, $gene if ($count{$gene} == 1);
			}
		if ($genLine =~ "\/product")
			{
			$product = $genLine;
				$product =~ s/"//g;
				$product =~ s/^\s+//;
				$product =~ s/\/product=//;
			$iGene{$product}++;
			$count{$product}++ if ($iGene{$product} == 1);
			push @GeneList, $product if ($count{$product} == 1);
			}
		}
close GENE;

@sortedGene = sort {$count{$b} <=> $count{$a} } keys %count;

open (OUT, ">GeneSummary.txt") or die "Cannot open results file, GeneSummary.txt.\n";
foreach $entry (@sortedGene)
	{
	print OUT "$entry\t$count{$entry}\n";
	}
close OUT;

# Subroutines used in the script

sub setLineBreak	# Check line breaks of input files and set input record separator accordingly
	{
	my $inFile = shift;
	$/ ="\n";
	open (IN, "<$inFile") or die "Cannot open $inFile to check form of line breaks.\n";
		while (<IN>)
			{
			if ($_ =~ /\r\n/)
				{
				print "\tDOS line breaks detected ...\n" if ($verbose);
				$/ ="\r\n";
				last;
				}
			elsif ($_ =~ /\r/)
				{
				print "\tMac line breaks detected ...\n" if ($verbose);
				$/ ="\r";
				last;
				}
			else
				{
				print "\tUnix line breaks detected ...\n" if ($verbose);
				$/ ="\n";
				last;
				}
			}
	close IN;
	}

